(* Mathematica Test File *)

(*	Worker Routines		************************************************************************ *)

(*	A simplification routine
*)
FullSimplifyAss[exp_] :=
	FullSimplify[exp, Assumptions->m > 0 && \[HBar] > 0 && \[Omega] > 0] // Expand // PowerExpand

(*	The Hamiltonian
*)	
Operator[H] = 1/2/m Operator[p]^2 + m \[Omega]^2/2 Operator[x]^2

(*	The Commutation Relation between Location and Momentum
*)
QASet[Commutator[Operator[x], Operator[p]] = I \[HBar] Operator[1]]

(*	x is Hermitian
*)
Operator[x, "H"] = Operator[x]

(*	p is Hermitian
*)
Operator[p, "H"] = Operator[p]

(*	a basic rule for the Creation Operator
*)
rule = 
	Operator[a, "H"] -> 
	Sqrt[m \[Omega]/2/\[HBar]] Operator[x] - I/Sqrt[2 m \[HBar] \[Omega]] Operator[p] // FullSimplifyAss

(*	2 rules for an arbitrary Commutator solved after the Operator products
*)
commutatorRules[op1_Operator,op2_Operator] :=
	Module[{commutator = Commutator[op1, op2], rule1, rule2},
		rule1 = Solve[commutator == (commutator // QACommutatorExpand), op1 ** op2];
		rule2 = Solve[commutator == (commutator // QACommutatorExpand), op2 ** op1];
		Join[rule1[[1]],rule2[[1]]]
	]

(*	from basic rule derive a second rule for Annihilation Operator and unify them
*)
rule1 = {rule, Map[Hermitian[#]&, rule]} // FullSimplifyAss

(*	from rule1 derive rules for Location and Momentum
*)
rule2 = Solve[rule1 /. Rule->Equal, {Operator[x], Operator[p]}][[1]] // FullSimplifyAss

(*	apply Commutator Rules to Annihilation / Creation Operator
*)
rule3 = commutatorRules[Operator[a], Operator[a, "H"]][[1]]

(*	apply Commutator Rules to Annihilation / Creation Operator
*)
rule4 = commutatorRules[Operator[a], Operator[a, "H"]][[2]]

(*	define a Number Operator
*)
rule5 = Operator[N] -> Operator[a, "H"] ** Operator[a]

(*	define a Number Operator
*)
rule6 = Operator[a, "H"] ** Operator[a] -> Operator[N]

(*	derive the Commutator for Annihilation / Creation Operator
*)
rule7 = 
	Commutator[Operator[a], Operator[a, "H"]] -> 
	(Commutator[Operator[a], Operator[a, "H"]] /. rule1 // QACommutatorExpand)

(*	define Eigenstate - Eigenvalue - Relation
*)
QASet[Operator[N] ** Ket[n_] = n Ket[n]]

(*	******************************************************************************************** *)


Test[
	Commutator[Operator[x], Operator[p]],
	I \[HBar] Operator[1],
	TestID->"Harmonic Oscillator-20160515-G7M1T8"
]

Test[
	Commutator[Operator[p], Operator[x]],
	-I \[HBar] Operator[1],
	TestID->"Harmonic Oscillator-20160515-S6U6G3"
]

Test[
	Operator[x, "H"],
	Operator[x],
	TestID->"Harmonic Oscillator-20160515-M5R4N2"
]

Test[
	Hermitian[Operator[x]],
	Operator[x],
	TestID->"Harmonic Oscillator-20160515-V6Q2L2"
]

Test[
	Operator[p, "H"],
	Operator[p],
	TestID->"Harmonic Oscillator-20160515-L2D4A6"
]

Test[
	Hermitian[Operator[p]],
	Operator[p],
	TestID->"Harmonic Oscillator-20160515-V7C8T4"
]

Test[
	Operator[o1] ** Operator[o2] /. commutatorRules[Operator[o1], Operator[o2]][[1]],
	Commutator[Operator[o1], Operator[o2]] + Operator[o2] ** Operator[o1],
	TestID->"Harmonic Oscillator-20160515-N2L7U6"
]

Test[
	rule7,
	Commutator[Operator[a], Operator[a, "H"]] -> Operator[1],
	TestID->"Harmonic Oscillator-20160515-U8P7J7"
]

Test[
	(Operator[H] /. rule2 // QAPowerExpand // FullSimplifyAss) /. rule3 /. rule7 // QAExpandAll,
	\[Omega] \[HBar] Operator[a, "H"] ** Operator[a] + 1/2 \[Omega] \[HBar] Operator[1],
	TestID->"Harmonic Oscillator-20160515-X9N6J3"
]

Test[
	((Operator[N] ** Operator[a, "H"] ** Ket[n] /. rule5) /. rule3) /. rule6 /. rule7,
	(n + 1) Operator[a, "H"] ** Ket[n] // Expand,
	TestID->"Harmonic Oscillator-20160515-A1X2P8"
]

Test[
	((Operator[N] ** Operator[a] ** Ket[n] /. rule5) /. rule4) /. rule6 /. rule7,
	(n - 1) Operator[a] ** Ket[n] // Expand,
	TestID->"Harmonic Oscillator-20160515-Q5D9K6"
]

(*	TEST OUTPUT	***********************************************************************************
*)
(*
Test[
	commutatorRules[Operator[o1], Operator[o2]],
	0,
	TestID->"Harmonic Oscillator-20160515-P6K1X0"
]
*)