(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[      8755,        270]
NotebookOptionsPosition[      4461,        158]
NotebookOutlinePosition[      7540,        228]
CellTagsIndexPosition[      7497,        225]
WindowTitle->QuantumAlgebra Guide - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:QuantumAlgebra/guide/QuantumAlgebra"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["QuantumAlgebra Guide", "GuideTitle",
 CellID->942062912],

Cell["\<\
This guide contains introductionary information about the Quantum Algebra \
application.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell["Quantum Mechanical Base Objects", "GuideFunctionsSubsection",
 CellID->351933073],

Cell["\<\
The symbols listed below serve as Quantum Mechanical base objects. Whereas a \
BraKet is treated as a constant, this is not true for the other objects. The \
latter are non-constants with special treatment. Nevertheless there are \
special cases where a combination of Bra - Operator - Ket is treated as \
constant - for simplification purposes, as in Quantum Theory such a \
combination represents a constant.
Remark: With constant we do not mean a physical constant here but an \
expression which is evaluated to a quantity.\
\>", "GuideText",
 CellID->348045444],

Cell[TextData[{
 ButtonBox["Bra",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/ref/Bra"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " This symbol signifies the basic Bra object of the Dirac Bra - Ket - \
Notation"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 ButtonBox["Ket",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/ref/Ket"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " This symbol signifies the basic Ket object of the Dirac Bra - Ket - \
Notation"
}], "GuideText",
 CellID->1463276848],

Cell[TextData[{
 ButtonBox["BraKet",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/ref/BraKet"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " This symbol signifies the basic BraKet object as a result of the \
multiplication of Bra and Ket objects"
}], "GuideText",
 CellID->56750302],

Cell[TextData[{
 Cell[TextData[ButtonBox["Operator",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/ref/Operator"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "This symbol signifies the basic Quantum Mechanical Operator object "
}], "GuideText",
 CellID->769448525],

Cell["\t", "GuideDelimiterSubsection",
 CellID->311258892]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input Possibilities", "GuideFunctionsSubsection",
 CellID->155582371],

Cell["\<\
There are two possibilities to enter Quantum Mechanical base objects: You can \
enter the fullform of the objects or use an input alias. With input alias \
you'll get a two-dimensional template with placeholders, where you can enter \
required information.
Each time when you initiate the Needs command to load the Quantum Algebra \
package, you'll get a printout of the aliases available for input.\
\>", "GuideText",
 CellID->209193518]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[{
 ButtonBox["Quantum Algebra Overview",
  BaseStyle->"Link",
  ButtonData->"paclet:QuantumAlgebra/tutorial/QuantumAlgebraOverview"],
 " - Overview page with links to all documentation pages"
}], "GuideTutorial",
 CellID->1885805579],

Cell[TextData[ButtonBox["Quantum Algebra Basics",
 BaseStyle->"Link",
 ButtonData->
  "paclet:QuantumAlgebra/tutorial/QuantumAlgebraBasics"]], "GuideTutorial",
 CellID->806871991]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Quantum Algebra Modules",
 BaseStyle->"Link",
 ButtonData->
  "paclet:QuantumAlgebra/guide/QuantumAlgebraModules"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"QuantumAlgebra Guide - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2016, 5, 31, 18, 14, 38.9953305}", "context" -> 
    "QuantumAlgebra`", 
    "keywords" -> {"Bra Ket BraKet Operator Dirac Constant"}, "index" -> True,
     "label" -> "Quantum Algebra Package Guide", "language" -> "en", "paclet" -> 
    "QuantumAlgebra Package", "status" -> "None", "summary" -> 
    "This guide contains introductionary information about the Quantum \
Algebra application.", "synonyms" -> {}, "title" -> "QuantumAlgebra Guide", 
    "type" -> "Guide", "uri" -> "QuantumAlgebra/guide/QuantumAlgebraGuide"}, 
  "LinkTrails" -> "", "SearchTextTranslated" -> ""},
FrontEndVersion->"10.2 for Microsoft Windows (64-bit) (July 7, 2015)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell["Bra Ket Styles", "Section"], 
   Cell[" The cells below define certain styles needed for defining \
Bras&Kets.These styles serve to give the correct structural properties to \
Bras & Kets ", "Text"], 
   Cell[
    StyleData["BraKetArg"], SpanMinSize -> Automatic, StyleMenuListing -> 
    None, AdjustmentBoxOptions -> {BoxMargins -> {{0, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> True, Selectable -> True}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> "UnmatchedBracket"}], 
   Cell[
    StyleData["BraWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.2, -0.13}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["KetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.1, 0.2}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["BraKetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, 
    SpanMinSize -> 2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.25, 0.25}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["RIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.16, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}], 
   Cell[
    StyleData["LIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0, -0.1}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}]}, Visible -> False, FrontEndVersion -> 
  "10.2 for Microsoft Windows (64-bit) (July 7, 2015)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 75, 1, 3, "GuideColorBar"],
Cell[692, 24, 194, 5, 19, "LinkTrail"],
Cell[CellGroupData[{
Cell[911, 33, 62, 1, 47, "GuideTitle",
 CellID->942062912],
Cell[976, 36, 142, 4, 48, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[1143, 44, 87, 1, 30, "GuideFunctionsSubsection",
 CellID->351933073],
Cell[1233, 47, 575, 10, 100, "GuideText",
 CellID->348045444],
Cell[1811, 59, 269, 9, 20, "GuideText",
 CellID->203374175],
Cell[2083, 70, 270, 9, 20, "GuideText",
 CellID->1463276848],
Cell[2356, 81, 300, 9, 20, "GuideText",
 CellID->56750302],
Cell[2659, 92, 311, 9, 20, "GuideText",
 CellID->769448525],
Cell[2973, 103, 58, 1, 21, "GuideDelimiterSubsection",
 CellID->311258892]
}, Open  ]],
Cell[CellGroupData[{
Cell[3068, 109, 75, 1, 30, "GuideFunctionsSubsection",
 CellID->155582371],
Cell[3146, 112, 448, 8, 86, "GuideText",
 CellID->209193518]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3643, 126, 62, 1, 21, "GuideTutorialsSection",
 CellID->415694126],
Cell[3708, 129, 247, 6, 19, "GuideTutorial",
 CellID->1885805579],
Cell[3958, 137, 179, 4, 19, "GuideTutorial",
 CellID->806871991]
}, Open  ]],
Cell[CellGroupData[{
Cell[4174, 146, 62, 1, 30, "GuideMoreAboutSection",
 CellID->23220180],
Cell[4239, 149, 180, 4, 18, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[4434, 156, 23, 0, 41, "FooterCell"]
}
]
*)

(* End of internal cache information *)
