(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     11131,        315]
NotebookOptionsPosition[      6656,        200]
NotebookOutlinePosition[     10327,        285]
CellTagsIndexPosition[     10242,        280]
WindowTitle->QASeries - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:QuantumAlgebra/guide/QuantumAlgebra"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["QUANTUMALGEBRA PACKAGE PACLET SYMBOL", "PacletNameCell"], Cell[
    TextData[Cell[BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->False], {"\<\"QuantumAlgebra/ref/QASeries\"\>":>
      None, "\<\"Copy Mathematica url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["QuantumAlgebra/ref/QASeries"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/QuantumAlgebra/ref/\
QASeries.html"], StandardForm]], "Input", TextClipboardType -> "PlainText"]}, 
           Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "QuantumAlgebra/ref/QASeries", ".html"]], None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell["QASeries", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"QASeries", "[", 
       RowBox[{
        StyleBox["expr", "TI"], ",", 
        RowBox[{"{", 
         RowBox[{
          ButtonBox["A",
           BaseStyle->"Link",
           ButtonData->"paclet:ref/A"], ",", 
          ButtonBox["B",
           BaseStyle->"Link",
           ButtonData->"paclet:ref/B"], ",", 
          StyleBox["n", "TI"]}], "}"}]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]expands the operator expr depending only on A, in power \
series of A around the operator B up to order n. The operator B must be of \
the form c ",
     Cell[BoxData[
      RowBox[{
       ButtonBox["Operator",
        BaseStyle->"Link",
        ButtonData->"paclet:QuantumAlgebra/ref/Operator"], "[", "1", "]"}]], 
      "InlineFormula"],
     " or ",
     Cell[BoxData[
      RowBox[{
       ButtonBox["Operator",
        BaseStyle->"Link",
        ButtonData->"paclet:QuantumAlgebra/ref/Operator"], "[", "0", "]"}]], 
      "InlineFormula"],
     " (zero). "
    }]]},
   {"", Cell[TextData[{
     "QASeries[Exp[A]\[LineSeparator]** B ** Exp[-A],n] expands the first \
argument as a series expansion in terms of operators A and B up to n terms. ",
     Cell[BoxData[
      ButtonBox["QASeries",
       BaseStyle->"Link",
       ButtonData->"paclet:QuantumAlgebra/ref/QASeries"]], "InlineFormula"],
     " does not return a series object, but a polynomial on operator A."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}}],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->168170920],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->399111911],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Loads", " ", "the", " ", "application"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"Needs", "[", "\"\<QuantumAlgebra`\>\"", "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->381283919],

Cell[CellGroupData[{

Cell[BoxData["\<\"Application Quantum Algebra by Juergen Habelt\"\>"], "Print",
 CellID->377178595],

Cell[BoxData["\<\"This package is based on the work of Cesar Guerra \
G.,\"\>"], "Print",
 CellID->235279877],

Cell[BoxData["\<\"who developed the original package for Mathematica \
4.1/4.2\"\>"], "Print",
 CellID->19787518],

Cell[BoxData["\<\"For proper styling You should invoke SetNotebookStyles \
once.\"\>"], "Print",
 CellID->49830103]
}, Open  ]],

Cell[BoxData["\<\"ALIASES:\\nESC op ESC - Operator without index\\nESC hop \
ESC - Hermitian Operator without index\\nESC ket ESC - Ket without \
index\\nESC ketd ESC - Ket with subscript(s)\\nESC ketu ESC - Ket with \
superscript(s)\\nESC ketdu ESC - Ket with subscript(s) & superscript(s)\\nESC \
bra ESC - Ket without index\\nESC dbra ESC - Bra with subscript(s)\\nESC ubra \
ESC - Bra with superscript(s)\\nESC dubra ESC - Bra with subscript(s) & \
superscript(s)\\nESC (du)braket(du) ESC - Bra with (left and / or right) \
subscript(s) and / or superscript(s).\"\>"], "Output",
 ImageSize->{390, 220},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->103292620]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"QASeries - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2016, 5, 31, 18, 15, 15.3312013}", "context" -> 
    "QuantumAlgebra`", "keywords" -> {}, "index" -> True, "label" -> 
    "QuantumAlgebra Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "QuantumAlgebra Package", "status" -> "None", "summary" -> 
    "QASeries[expr, {A, B, n}] expands the operator expr depending only on A, \
in power series of A around the operator B up to order n. The operator B must \
be of the form c Operator[1] or Operator[0] (zero). QASeries[Exp[A] ** B ** \
Exp[-A],n] expands the first argument as a series expansion in terms of \
operators A and B up to n terms. QASeries does not return a series object, \
but a polynomial on operator A.", "synonyms" -> {}, "title" -> "QASeries", 
    "type" -> "Symbol", "uri" -> "QuantumAlgebra/ref/QASeries"}, "LinkTrails" -> 
  "", "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"10.2 for Microsoft Windows (64-bit) (July 7, 2015)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"], 
   Cell["Bra Ket Styles", "Section"], 
   Cell[" The cells below define certain styles needed for defining \
Bras&Kets.These styles serve to give the correct structural properties to \
Bras & Kets ", "Text"], 
   Cell[
    StyleData["BraKetArg"], SpanMinSize -> Automatic, StyleMenuListing -> 
    None, AdjustmentBoxOptions -> {BoxMargins -> {{0, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> True, Selectable -> True}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> "UnmatchedBracket"}], 
   Cell[
    StyleData["BraWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.2, -0.13}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["KetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, SpanMinSize -> 
    2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.1, 0.2}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["BraKetWrapper"], ScriptBaselineShifts -> {0.6, 0.9}, 
    SpanMinSize -> 2, StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0.25, 0.25}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}, 
    AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}], 
   Cell[
    StyleData["RIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{-0.16, 0}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}], 
   Cell[
    StyleData["LIndex"], StyleMenuListing -> None, 
    AdjustmentBoxOptions -> {BoxMargins -> {{0, -0.1}, {0, 0}}}, 
    TagBoxOptions -> {Editable -> False}]}, Visible -> False, FrontEndVersion -> 
  "10.2 for Microsoft Windows (64-bit) (July 7, 2015)", StyleDefinitions -> 
  "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[4480, 128, 461, 13, 30, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->168170920]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 10099, 273}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[602, 21, 76, 1, 3, "SymbolColorBar"],
Cell[681, 24, 194, 5, 19, "LinkTrail"],
Cell[878, 31, 1894, 42, 48, "AnchorBarGrid",
 CellID->1],
Cell[2775, 75, 51, 1, 43, "ObjectName",
 CellID->1224892054],
Cell[2829, 78, 1626, 46, 170, "Usage"],
Cell[CellGroupData[{
Cell[4480, 128, 461, 13, 30, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->168170920],
Cell[CellGroupData[{
Cell[4966, 145, 148, 5, 25, "ExampleSection",
 CellID->399111911],
Cell[CellGroupData[{
Cell[5139, 154, 252, 7, 47, "Input",
 CellID->381283919],
Cell[CellGroupData[{
Cell[5416, 165, 99, 1, 23, "Print",
 CellID->377178595],
Cell[5518, 168, 109, 2, 23, "Print",
 CellID->235279877],
Cell[5630, 172, 113, 2, 23, "Print",
 CellID->19787518],
Cell[5746, 176, 115, 2, 23, "Print",
 CellID->49830103]
}, Open  ]],
Cell[5876, 181, 714, 12, 241, "Output",
 CellID->103292620]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[6629, 198, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)
